//--------------------------------------------------------------
// Press F1 to get help about using script.
// To access an object that is not located in the current class, start the call with Globals.
// When using events and timers be cautious not to generate memoryleaks,
// please see the help for more information.
//---------------------------------------------------------------
	
    /*
	*******************************************************************
	All examples in this support document are only intended
	to improve understanding of the functionality and 
	handling of the equipment. 
	Beijer Electronics AB cannot assume any liability if these
	examples are used in real applications.
	In view of the wide range of applications for this 
	equipment, users must acquire sufficient knowledge 
	themselves in order to ensure that it is correctly used in 
	their specific application. Persons responsible for the 
	application and the equipment must themselves 
	ensure that each application is in compliance with all 
	relevant requirements, standards and legislation in 
	respect to configuration and safety.
	Beijer Electronics AB will accept no liability for any 
	damage incurred during the installation or use of this 
	equipment.
	Beijer Electronics AB prohibits all modification, changes
	or conversion of the equipment.
	*******************************************************************
	*/

namespace Neo.ApplicationFramework.Generated
{
	using System.Windows.Forms;
	using System;
	using System.Drawing;
	using Neo.ApplicationFramework.Tools;
	using Neo.ApplicationFramework.Common.Graphics.Logic;
	using Neo.ApplicationFramework.Controls;
	using Neo.ApplicationFramework.Interfaces;
	using System.Xml;
	using System.IO;
	using System.Xml.Linq;
	using System.Text;
    
	public partial class XMLScriptModule
	{
		#region Constants
		// The url to MSDN documentation of the XDocument
		private const string URL_Msdn = "http://msdn.microsoft.com/en-us/library/system.xml.linq.xdocument.aspx";

		// The name of the root element for the XML. 
		private const string STR_RootElement = "RootElement";
		// The name of text elements for the XML. 
		private const string STR_TextElement = "TextElement";
		// The attribute name for creation time for the text. 
		private const string STR_Created = "Created";
		// The attribute name for the elements for the XML. 
		private const string STR_ElementType = "ElementType";
		// The element type for text elements. 
		private const string STR_ElementTypeText = "Text";
		#endregion

		#region Public Methods
		/// <summary>
		/// Writes a text to an XML file.
		/// </summary>
		/// <param name="path">The full path to the file.</param>
		/// <param name="text">The Text to append.</param>
		/// <param name="append">If false, creates a new file.</param>
		public void WriteText(string path, string text, bool append)
		{
			// Checks if the file exists, and if the user 
			// doesn't want to append to an already existing file. 
			if (!append && File.Exists(path))
			{
				// Deletes the file.
				File.Delete(path);
			}

			// Retrieves a XDocument.
			XDocument xDocument = GetXDocument(path);
			// Appends the text to the root element.
			AppendTextToRootElement(text, xDocument);
			// Saves the XML to the specified path.
			xDocument.Save(path);
		}

		/// <summary>
		/// Reads and returns the text from the specified path.
		/// </summary>
		/// <param name="path">The full path to the file.</param>
		/// <returns>A string with the text.</returns>
		public string ReadText(string path)
		{
			// First checks if the file exists.
			// If it doesn't, it returns an empty string.
			if (!File.Exists(path))
				return string.Empty;

			// Retrieves a XDocument.
			XDocument xDocument = GetXDocument(path);
			// Tries to get the root element of the XML.
			XElement rootElement = xDocument.Element(STR_RootElement);
			// If found, iterate it's children and return all the text.
			if (rootElement != null)
				return IterateChildNodesAndGetTexts(rootElement);

			// If no root element was found, return an empty string.
			return string.Empty;
		}

		/// <summary>
		/// Reads and returns the XML from the specified path.
		/// </summary>
		/// <param name="path">The full path to the file.</param>
		/// <returns>A string with the XML.</returns>
		public string ReadXML(string path)
		{
			// First checks if the file exists.
			// If it doesn't, it returns an empty string.
			if (!File.Exists(path))
				return string.Empty;

			// Retrieves a XDocument.
			XDocument xDocument = GetXDocument(path);
			// Returns the XML as text.
			return xDocument.ToString();
		}

		#endregion

		#region XmlDocument Creation
		/// <summary>
		/// Loads or creates a XDocument.
		/// </summary>
		/// <param name="path">Full path to a file if loading is the purpose.</param>
		/// <returns>A new XDocument if path is empty or invalid, otherwise a saved XDocument.</returns>
		private static XDocument GetXDocument(string path)
		{
			// If the path is empty or there is no file with that path,
			// create a new XDocument with a root element and return it.
			if (string.IsNullOrEmpty(path) || !File.Exists(path))
				return CreateXDocumentWithRootElement();

			// If the path is valid, load the XDocument and return it.
			return LoadXDocument(path);
		}

		/// <summary>
		/// Creates a new XDocument.
		/// </summary>
		/// <returns>new XDocument with a root element and a comment.</returns>
		private static XDocument CreateXDocumentWithRootElement()
		{
			// Creates a XDocument, and adds a comment and an element to it. 
			XDocument xDocument = new XDocument(
				new XComment("This is a comment"),
				new XElement(STR_RootElement)
				);

			// Returns the newly created XDocument.
			return xDocument;
		}

		/// <summary>
		/// Loads a XDocument.
		/// If it goes wrong, it will return a new XDocument.
		/// </summary>
		/// <param name="path">Full path to the file.</param>
		/// <returns>Loaded or new XDocument.</returns>
		private static XDocument LoadXDocument(string path)
		{
			XDocument xDocument;
			// Uses try/catch to prevent termination
			// of the program, should anything go wrong.
			try
			{
				// Try to load the file.
				xDocument = XDocument.Load(path);
			}
			catch (Exception)
			{
				// If the loading went wrong, create a new XDocument.
				xDocument = CreateXDocumentWithRootElement();
			}
			// Return teh document.
			return xDocument;
		}

		#endregion

		#region Getting data from Xml
		/// <summary>
		/// Iterates the children of an element.
		/// Retrieving the texts of the children that
		/// has the correct attribute, and then 
		/// returning the texts.
		/// </summary>
		/// <param name="element">The element to iterate over.</param>
		/// <returns>A string with the texts found.</returns>
		private static string IterateChildNodesAndGetTexts(XElement element)
		{
			// Create a StringBuilder to hold the result.
			StringBuilder textStrings = new StringBuilder();

			// If the element doesn't have children,
			// we can abort and return an empty string.
			if (!element.HasElements)
				return textStrings.ToString();

			// Iterate through the children of the element.
			foreach (XElement childNode in element.Elements())
			{
				// Check if the child node has the Text attribute.
				if (HasTextAttribute(childNode))
				{
					// If so, add the text value to the result.
					textStrings.AppendLine(childNode.Value);
				}
			}

			// Return the result.
			return textStrings.ToString();
		}

		/// <summary>
		/// Checks if a element has a type attribute,
		/// and that the attribute is the Text attribute.
		/// </summary>
		/// <param name="element">Element to check.</param>
		/// <returns>True if it was the correct attribute and value, otherwise false.</returns>
		private static bool HasTextAttribute(XElement element)
		{
			// Try to get the ElementType attribute from the element.
			XAttribute attribute = element.Attribute(STR_ElementType);
			// If it was found and has the Text type value, it will return true.
			return (attribute != null && attribute.Value == STR_ElementTypeText);
		}

		#endregion

		#region Applying data to Xml
		/// <summary>
		/// Appends a text to the root element of a given XDocument.
		/// </summary>
		/// <param name="text">The text to append.</param>
		/// <param name="xDocument">The XDocument to append to.</param>
		private static void AppendTextToRootElement(string text, XDocument xDocument)
		{
			// Try to get the root element.
			XElement rootElement = xDocument.Element(STR_RootElement);
			// If the document didn't have the root element, abort.
			if (rootElement == null)
				return;
            
			// Create a new element to store the text in.
			XElement textElement = new XElement(STR_TextElement);
			// Create a attribute to hold the creation time of the text, 
			// and set the value to the current date and time.
			XAttribute createdAttribute = new XAttribute(STR_Created, DateTime.Now.ToString());
			// Create a attribute for the element type and set it.
			XAttribute typeAttribute = new XAttribute(STR_ElementType, STR_ElementTypeText);

			// Add the attributes to the element.
			textElement.Add(createdAttribute);
			textElement.Add(typeAttribute);
			// Store the text in the element.
			textElement.Value = text;

			// Add the text element to the root element.
			rootElement.Add(textElement);
		}

		#endregion

	}
}
